using FitConnect.Models;
using FitConnect.Models.v1.Api;
using FitConnect.Services.Models;

namespace FitConnect.Interfaces.Subscriber;

public interface ISubscriberWithSubmission : WithSubmission {
    public Submission? Submission { get; }

    /// <summary>
    ///     Returns the data (Fachdaten) of the submission
    /// </summary>
    /// <returns></returns>
    public string? GetDataJson() {
        return Submission?.Data;
    }

    /// <summary>
    ///     Loads the <see cref="Attachment" />s for the given <see cref="Submission" />.
    /// </summary>
    /// <returns>Enumerable of the attachments</returns>
    public Task<IEnumerable<Attachment>> GetAttachmentsAsync();

    /// <summary>
    ///     Accept submission and delete it from the server
    /// </summary>
    public Task AcceptSubmissionAsync();

    /// <summary>
    ///     Rejects the submission
    /// </summary>
    /// <param name="problems">Reasons for the rejection</param>
    public Task RejectSubmissionAsync(params Problems[] problems);

    // /// <summary>
    // ///     Set submission state to forwarded
    // /// </summary>
    // public void ForwardSubmission();

    /// <summary>
    ///     Set submission state
    /// </summary>
    /// <param name="status">state the submission has to be set to</param>
    public Task CompleteSubmission(FinishSubmissionStatus status);
}

public interface WithSubmission {
    /// <summary>
    /// Verifies the submission acceptance status
    /// </summary>
    /// <param name="acceptanceStatus"></param>
    /// <returns></returns>
    public bool VerifyStatus(AcceptanceStatus acceptanceStatus);
}
